﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ALPS {
  [Item("LayerSorter", "")]
  [StorableClass]
  public class LayerSorter : SingleSuccessorOperator {
    private IScopeTreeLookupParameter<IntValue> LayerParameter {
      get { return (IScopeTreeLookupParameter<IntValue>)Parameters["Layer"]; }
    }

    [StorableConstructor]
    private LayerSorter(bool deserializing)
      : base(deserializing) { }
    private LayerSorter(LayerSorter original, Cloner cloner)
      : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new LayerSorter(this, cloner);
    }
    public LayerSorter()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<IntValue>("Layer"));
    }

    public override IOperation Apply() {
      var layers = ExecutionContext.Scope.SubScopes;
      layers.Sort(SortLayers);

      return base.Apply();
    }

    private int SortLayers(IScope x, IScope y) {
      var lx = (IntValue)x.Variables["Layer"].Value;
      var ly = (IntValue)y.Variables["Layer"].Value;

      return lx.Value.CompareTo(ly.Value);
    }
  }
}